/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: lockfile.hxx,v $
 *
 *  $Revision: 1.11 $
 *
 *  last change: $Author: obo $ $Date: 2006/01/19 18:02:49 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

/* Information:
 * This class implements a mechanism to lock a users installation directory,
 * which is necessesary because instances of staroffice could be running on 
 * different hosts while using the same directory thus causing data 
 * inconsistency.
 * When an existing lock is detected, the user will be asked whether he wants
 * to continue anyway, thus removing the lock and replacing it with a new one
 *
 * ideas:
 * - store information about user and host and time in the lockfile and display 
 * these when asking whether to remove the lockfile.
 * - periodically check the lockfile and warn the user when it gets replaced
 *
 */

#ifndef _SAL_TYPES_H_
#include "sal/types.h"
#endif
#ifndef _RTL_USTRING_HXX_
#include "rtl/ustring.hxx"
#endif

class ByteString;

namespace desktop {

	class Lockfile;
	bool Lockfile_execWarning( Lockfile * that );

	class Lockfile
	{
	public:

		// contructs a new lockfile onject
		Lockfile( bool bIPCserver = true );

        // separating GUI code:
        typedef bool (* fpExecWarning)( Lockfile * that );
        
		// checks the lockfile, asks user when lockfile is 
		// found (iff gui) and returns false when we may not continue
		sal_Bool check( fpExecWarning execWarning );

		// removes the lockfile. should only be called in exceptional situations
		void clean(void);

		// removes the lockfile
		~Lockfile(void);

	private:
		// data in lockfile
		static const ByteString Group();
		static const ByteString Userkey();
		static const ByteString Hostkey();
		static const ByteString Stampkey();
		static const ByteString Timekey();
		static const ByteString IPCkey();
		// lockfilename
		static const rtl::OUString Suffix();
		bool m_bIPCserver;
		// full qualified name (file://-url) of the lockfile
		rtl::OUString m_aLockname;
		// flag whether the d'tor should delete the lock
		sal_Bool m_bRemove;
		sal_Bool m_bIsLocked;
		// ID 
		rtl::OUString m_aId;
		rtl::OUString m_aDate;
		// access to data in file
		void syncToFile(void) const;
		sal_Bool isStale(void) const;
        friend bool Lockfile_execWarning( Lockfile * that );

	};

}
